<?php
/**
 * @package	Speedup Optimization
 * @copyright	Copyright (C) 2022 SiteGuarding.com. All rights reserved.
 * @GNU General Public License
 */

    function speedup01_ajax_enabled()
    {
        ini_set('max_execution_time',300);

        if (speedup01_Is_SpeedUp_module_Installed())
        {
            $status = isset($_POST['status']) ? trim($_POST['status']) : '';

            $answer = speedup01_module_switcher($status);
        }
        else {
            // SpeedUp is not installed
            $answer = array(
                'msg' => 'Please complete installation and activation process',
                'redirect' => $_SERVER['REQUEST_URI'],
            );
        }


        echo json_encode($answer);

    }

    function speedup01_module_switcher($status) 
    {
            
        $file_config = dirname(DIR_APPLICATION).'/webanalyze/firewall/firewall.config.php';
        $file_on = dirname(DIR_APPLICATION).'/webanalyze/firewall/firewall.speedup.on';
        $file_off = dirname(DIR_APPLICATION).'/webanalyze/firewall/firewall.speedup.off';
        
        switch ($status)
        {
            case 'disable':
                speedup01_ChangeFirewallConfig('SITEGUARDING_SPEEDUP', false);
                @unlink($file_on);
                speedup01_SaveFile($file_off, '');
                $action_result_txt = 'Optimization - Disabled';
                break;
                
            case 'enable':
                speedup01_ChangeFirewallConfig('SITEGUARDING_SPEEDUP', true);
                @unlink($file_off);
                speedup01_SaveFile($file_on, '');
                $action_result_txt = 'Optimization - Activated';
                break;
        }
        
        $answer = array(
            'msg' => $action_result_txt,
            'redirect' => '',
        );
        
        return $answer;
    }



    /*
    Purge All
    */

    function speedup01_ajax_purge_all() 
    {
        ini_set('max_execution_time',300);
        
        if (speedup01_Is_SpeedUp_module_Installed())
        {
            // Purge cache of speedup 
            $result = speedup01_purge_cache();
            
            $total_size = $result['total_size'];
            $total_files = $result['total_files'];
            
            $answer = array(
                'msg' => 'Cache has beed purged (size: '.$total_size.' bytes, files: '.$total_files.')',
                'redirect' => '',
            );
        }
        else {
            // SpeedUp is not installed
            $answer = array(
                'msg' => 'Please complete installation and activation process',
                'redirect' => $_SERVER['REQUEST_URI'],
            );
        }
        

        echo json_encode($answer);

    }
    
    function speedup01_purge_cache() 
    {
        $cache_path = dirname(DIR_APPLICATION).'/webanalyze/firewall/speedup_cache';
        
        $total_size = 0;
        $total_files = 0;
        foreach (glob($cache_path."/*.*") as $filename) 
        {
        	$short_filename = basename($filename);
        	if ($short_filename == '.htaccess') continue;
        	
        	$total_size += filesize($filename);
        	$total_files++;
        	
        	unlink($filename);
        }
            
        $result['total_size'] = $total_size;
        $result['total_files'] = $total_files;
        
        return $result;
    }




    
    function speedup01_Is_SpeedUp_module_Installed()
    {
        speedup01_Install_core_files();
        
        $file = dirname(DIR_APPLICATION).'/webanalyze/firewall/firewall.speedup.php';
        if (file_exists($file)) return true;
        else return false;
    }
    
    
    function speedup01_Install_core_files()
    {
        // Check for webanalyze folder
        $folder = dirname(DIR_APPLICATION).'/webanalyze';
        if (!file_exists($folder)) mkdir($folder);
        $folder .= '/firewall';
        if (!file_exists($folder)) mkdir($folder);
        
        // Check for all files
        $file = $folder.'/firewall.speedup.php';
        if (!file_exists($file)) copy(dirname(__FILE__).'/core/firewall.speedup.php', $file);
        
        $file = $folder.'/firewall.speedup.ini';
        if (!file_exists($file)) copy(dirname(__FILE__).'/core/firewall.speedup.ini', $file);
        
        
        // Check index.php
        $index_file = dirname(DIR_APPLICATION).'/index.php';
        $content = speedup01_LoadFile($index_file);
        if (strpos($content, '21D467F52F85') === false)
        {
            // Need to insert TOP
            $insert_TOP = true;
        }
        else $insert_TOP = false;
        
        if (strpos($content, '555092211981') === false)
        {
            // Need to insert END
            $insert_END = true;
        }
        else $insert_END = false;
        
        if ($insert_TOP || $insert_END)
        {
            if ($insert_TOP) $content = '<?php 
// OPTIMIZATION START - TOP - Added by SpeedUp (21D467F52F85)
$SG_SPEEDUP_CLASS_FILE = dirname(__FILE__)."/webanalyze/firewall/firewall.speedup.php";
if (file_exists($SG_SPEEDUP_CLASS_FILE)) {
require_once($SG_SPEEDUP_CLASS_FILE);
$SG_SpeedUp = new SG_SpeedUp();
$SG_SpeedUp->Start();
} else die("Error! Absent: ".$SG_SPEEDUP_CLASS_FILE);
// OPTIMIZATION END - TOP - Added by SpeedUp
?>'.$content;
            if ($insert_END)
            {
                if (substr($content, -2) == '?>') $content = $content.'<?php ';
                $content = $content.' 
// OPTIMIZATION START - BOTTOM - Added by SpeedUp (555092211981)
if (class_exists("SG_SpeedUp")) $SG_SpeedUp->Finalize();
// OPTIMIZATION END - BOTTOM - Added by SpeedUp
';
            } 
            
            speedup01_SaveFile($index_file, $content);
        }
        


        // Check .htaccess
        $htaccess_file = dirname(DIR_APPLICATION).'.htaccess';
        $content = speedup01_LoadFile($htaccess_file);
        if (strpos($content, 'C94FEE55EE47') === false)
        {
            $content = '# OPTIMIZATION - Added by SpeedUp (C94FEE55EE47)

# Below is custom site speed items
# We need to set correct content-type for fonts
  AddType application/vnd.ms-fontobject .eot 
  AddType application/x-font-ttf .ttf
  AddType application/x-font-opentype .otf
  AddType application/x-font-woff .woff
  AddType image/svg+xml .svg


<IfModule expires_module>
    ExpiresActive on
    ExpiresByType text/plain "access plus 1 year"
    ExpiresByType text/css "access plus 1 year"
    ExpiresByType text/javascript "access plus 1 year"
    ExpiresByType application/javascript "access plus 1 year"
    ExpiresByType application/x-javascript "access plus 1 year"
    ExpiresByType image/svg+xml "access plus 1 year"
    ExpiresByType image/gif "access plus 1 year"
    ExpiresByType image/png "access plus 1 year"
    ExpiresByType image/ico "access plus 1 year"
    ExpiresByType image/x-icon "access plus 1 year"
    ExpiresByType image/jpg "access plus 1 year"
    ExpiresByType image/jpe "access plus 1 year"
    ExpiresByType image/jpeg "access plus 1 year"
    ExpiresByType font/truetype "access plus 1 year"
    ExpiresByType application/x-font-ttf "access plus 1 year"
    ExpiresByType font/opentype "access plus 1 year"
    ExpiresByType application/x-font-otf "access plus 1 year"
    ExpiresByType application/font-woff "access plus 1 year"
    ExpiresByType application/vnd.ms-fontobject "access plus 1 year"
    ExpiresByType application/x-shockwave-flash "access plus 1 year"
    ExpiresByType application/x-httpd-php-source "access plus 1 year"
# This part sets the expires for the fonts
  ExpiresByType application/vnd.ms-fontobject "access plus 1 year"
  ExpiresByType application/x-font-ttf "access plus 1 year"
  ExpiresByType application/x-font-opentype "access plus 1 year"
  ExpiresByType application/x-font-woff "access plus 1 year"
  ExpiresByType image/svg+xml "access plus 1 year"
</IfModule>


# This sets up browser cache control
# BEGIN Cache-Control Headers
<ifModule mod_headers.c>
  <filesMatch "\.(ico|jpe?g|png|gif|swf)$">
    Header set Cache-Control "public"
  </filesMatch>
  <filesMatch "\.(css)$">
    Header set Cache-Control "public"
  </filesMatch>
  <filesMatch "\.(js)$">
    Header set Cache-Control "private"
  </filesMatch>
  <filesMatch "\.(x?html?|php)$">
    Header set Cache-Control "private, must-revalidate"
  </filesMatch>
</ifModule>
# END Cache-Control Headers

# This is for gzip, which compresses files
<ifModule mod_gzip.c>
  mod_gzip_on Yes
  mod_gzip_dechunk Yes
  mod_gzip_item_include file .(html?|txt|css|js|php|pl)$
  mod_gzip_item_include handler ^cgi-script$
  mod_gzip_item_include mime ^text/.*
  mod_gzip_item_include mime ^application/x-javascript.*
  mod_gzip_item_exclude mime ^image/.*
  mod_gzip_item_exclude rspheader ^Content-Encoding:.*gzip.*
</ifModule>

# OPTIMIZATION - END - Added by SpeedUp


'.$content;
            speedup01_SaveFile($htaccess_file, $content);
        
        }
    }



    function speedup01_ChangeFirewallConfig($search_value, $set_value)
    {
        $file_config = dirname(DIR_APPLICATION).'/webanalyze/firewall/firewall.config.php';
        
        if (!file_exists($file_config)) return; // Just WP plugin is installed

        $fp = fopen($file_config, "r");
        $content = fread($fp, filesize($file_config));
        fclose($fp);
        
        if (trim($content) != "")
        {
            $content = speedup01_ChangeFirewallConfig_value($content, $search_value, $set_value);
            
            $fp = fopen($file_config, 'w');
            fwrite($fp, $content);
            fclose($fp);
        }
    }
    
    function speedup01_ChangeFirewallConfig_value($content, $search_value, $set_value)
    {
        if (trim($content) == '') return false;
        
        // Check for new contants
        if (stripos($content, $search_value) === false)
        {
            if (is_bool($set_value))
            {
                if ($set_value === true || $set_value == 1) $set_value_txt = 'true';
                if ($set_value === false || $set_value == 0) $set_value_txt = 'false';
                
                $new_line = "\n"."define( '".$search_value."', ".$set_value_txt.");"."\n";
            }
            else {
                $new_line = "\n"."define( '".$search_value."', '".$set_value."');"."\n";
            }
            
            // New value is absent , add new line
            if (stripos($content, '?>') === false)
            {
                // Add line in the end of file
                $content = $content.$new_line;
            }
            else {
                // Replace method in the end befor close php tag
                $content = str_ireplace('?>', $new_line, $content);
            }
            
            return $content;
        }
        else {
            // Replace in exist contants 
            $content_arr = explode("\n", $content);
            foreach ($content_arr as $k => $row)
            {
                $pos_i = stripos($row, $search_value);
                if ($pos_i !== false && $pos_i < 25)
                {
                    if (is_bool($set_value))
                    {
                        if ($set_value === true || $set_value == 1) $set_value_txt = 'true';
                        if ($set_value === false || $set_value == 0) $set_value_txt = 'false';
                        
                        $row = "define( '".$search_value."', ".$set_value_txt.");";
                    }
                    else {
                        $row = "define( '".$search_value."', '".$set_value."');";
                    }
                    
                    $content_arr[$k] = $row;
                }
            }
            
            $content = implode("\n", $content_arr);
            
            return $content;
        }
    }
    
    

    
    
    function speedup01_SaveDebug( $txt ) 
    {
        $log_file = dirname(__FILE__).'/debug.log';
        $fp = fopen($log_file, 'a');
        $line = date("Y-m-d H:i:s").' '.$txt."\n";
        fwrite($fp, $line);
        fclose($fp);
    }
    
    function speedup01_LoadFile( $file ) 
    {
        $contents = '';
        
        if (file_exists($file))
        {
            $fp = fopen($file, "r");
            $contents = fread($fp, filesize($file));
            fclose($fp); 
        }
        
        return $contents;
    }
    
    
    function speedup01_SaveFile( $file, $content, $append_flag = false ) 
    {
        if ($append_flag == true) $fp = fopen($file, 'a');
        else $fp = fopen($file, 'w');
        if ($fp === false) return false;
        
        $a = fwrite($fp, $content);
        if ($a === false) return false;
        
        fclose($fp);
        
        return true;
    }
    

    
    function speedup01_ServiceURL()
	{
	    $service_links = array(
            'https://',
            'www.',
            'site',
            'guard',
            'ing.com'
        );
        return implode("", $service_links);
	}
	
